*FILE MEMBER=muon_cosmic_flux_surface LIBRARY=SNOMAN LANGUAGE=FORTRAN77
*FILE DATE=15:Jan:2003
*
      SUBROUTINE MUON_COSMIC_FLUX_SURFACE(EMUON,THETA,PHI,WEIGHT)
*
*     SNOMAN: Calculate cosmic ray flux at the surface.
*
*
*     Contact:  J. Formaggio (UW)
*     Verified:
*     Refereed:
*
*     Parameters:
*     ==========
*
*           emuon,theta,phi:           <out>
*                      muon energy (GeV) and direction
*           weight:                    <out>
*                      probability of event.
*
*     Common Block Access:-
*     ===================
*     Cosmics and constants
*
*     Specification:-
*     =============
*     This code uses Gaisser's muon flux at the surface to find energy
*     and angular distribution.
*
*     Program Notes:-
*     =============
*
*     Revision History:-
*     ================
*
*     4.02   J. Formaggio  First version.
*            N. West       Remove unused variables.
 
*     Argument Declarations:-
*     =====================
 
      IMPLICIT NONE
 
C     Include files
 
      INCLUDE "cosmics.inc"
 
*ENDHEADER
************************************************************************
 
C     Local variables
 
      LOGICAL FIRST
      DOUBLE PRECISION EMUON, THETA, PHI, WEIGHT
      DOUBLE PRECISION DIDE, EMU
      DOUBLE PRECISION COSTH, I_TOT, DE, DCOSTH
      DOUBLE PRECISION LEMU, JACOBIAN, COSMIC_FLUX
      REAL RANUM
      DATA FIRST /.TRUE./
      SAVE FIRST
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO = 0.D0)
      EXTERNAL RANUM

      DOUBLE PRECISION PI, TWOPI
      PARAMETER (PI = 3.141592654)
      PARAMETER (TWOPI = 2. * PI)

 
C     First, integrate muon flux to find normalization.
 
      I_TOT    = ZERO
      IF (FIRST) THEN
         FIRST = .FALSE.
         DE = (LEMAX-LEMIN)/1000.
         DCOSTH = 1./1000.
         DO LEMU = LEMIN, LEMAX, DE
            EMU = 10.**LEMU
            JACOBIAN = LOG(10.)*EMU
            DO COSTH = 0. , 1., DCOSTH
               DIDE = COSMIC_FLUX(EMU,COSTH)
               I_TOT = I_TOT + JACOBIAN * DIDE * DE * DCOSTH
            ENDDO
         ENDDO
         XS_COSMIC_TOTAL = I_TOT
         WRITE(6,*) 'COSMIC FLUX = ',XS_COSMIC_TOTAL,'/cm-sq/s'
      ENDIF
 
C     For each event, throw the direction, energy, and associated weight
 
      PHI = TWOPI * RANUM(0)
      LEMU = LEMIN + RANUM(0)*(LEMAX-LEMIN)
      EMUON = 10.**(LEMU)
      JACOBIAN = LOG(10.)*EMUON
      COSTH = CTMIN + (CTMAX-CTMIN)*RANUM(0)
      THETA = DACOS(COSTH)

      WEIGHT = 4. * PI * JACOBIAN * COSMIC_FLUX(EMUON,COSTH) 
 
      RETURN
      END
 
      DOUBLE PRECISION FUNCTION COSMIC_FLUX(EMU,COSTH)
 
      IMPLICIT NONE
 
      INCLUDE "cosmics.inc"
 
      DOUBLE PRECISION EMU,COSTH,PAR(2)
      DATA PAR /115., 850./
 
C     This function returns DI/Dcos*DE for muons at the Earth's surface.
 
      COSMIC_FLUX = 0.
      IF ((COSTH.LE.CTMIN).OR.(COSTH.GT.CTMAX)) RETURN
 
      IF (MODEL.EQ.0) THEN

         COSMIC_FLUX = CS_A * 0.14 * EMU **(-CS_GAMMA) *
     $        (1./(1.+1.1*EMU*COSTH/PAR(1)) +
     $        0.054/(1.+1.1*EMU*COSTH/PAR(2)) + CS_RC)
         
      ELSEIF (MODEL.EQ.1) THEN

         COSMIC_FLUX = CS_A * 372. / (EMU*COSTH + 80.) *
     $        (5.*EMU + 10./COSTH)**(-CS_GAMMA) *
     $        ((EMU + 15.)/(EMU + 15./COSTH))

      ENDIF

      RETURN
      END
*endfile MEMBER=muon_cosmic_flux_surface
