*FILE MEMBER=muon_music LIBRARY=SNOMAN LANGUAGE=FORTRAN77
*FILE DATE=01:Jan:2002
*
      SUBROUTINE MUON_MUSIC(Emu,zf,Emu_f,x,y,z,t0,theta,phi,dr,t10)
*
*! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ !
*! MUSIC: MUon SImulation Code
*! by P. Antonioli, C. Ghetti, E.V. Korolkova,
*! V.A. Kudryavtsev, G. Sartorelli
*!
*! For any problems please contact:
*!
*! antonioli@bo.infn.it
*! v.kudryavtsev@sheffield.ac.uk
*! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ !
*! MUSIC is a 3D muon propagation code:
*! it takes into account energy losses and deflections
*! due breemstrahlung, pair production, inelastic scattering
*! and ionisation.
*! It simulates also the angular and lateral deflections due to
*! multiple scattering
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C       V2(100) - ARRAY OF LOG OF RELATIVE ENERGY TRANSFERS (E/EMU) FOR WHICH
C               THE MUON INTERACTION INTEGRAL CROSS-SECTIONS WERE CALCULATED:
C               FROM 0.0 TO -5.0 WITH THE STEP OF -0.05. THE INTEGRATION STARTS
C               FROM FROM V2=ALOG10(E/EMU)=0.0 AND ENDS AT V2(I).
C       CS1(100,71,4) - ARRAY OF NORMALIZED INTEGRAL CROSS-SECTIONS FOR V2(50),
C               LOG OF MUON ENERGY (61) (FROM 0.0 (1 GEV) TO 6.0 (1000 TEV)
C               WITH THE STEP OF 0.1) AND 4 PROCESSES TAKING INTO ACCOUNT
C               (BREEMSTRAHLUNG, PAIR PRODUCTION, INELASTIC SCATTERING
C               AND IONISATION)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
 
      DOUBLE PRECISION mmu
      REAL yfl, ranum
      INTEGER err
      INTEGER ms_flag,d_flag
      PARAMETER (mmu=0.105655d0)
      PARAMETER (ms_flag=1)   !to switch off mult.scatt. put ms_flag=0
      PARAMETER (d_flag=1)    !to switch off scatt. due to other processes
                              ! put d_flag=0
      INCLUDE "cosmics.inc"
 
*       ema(61) - array of log of muon energies (GeV), 0.0, 0.1,...6.0.
*       va(30) - array of log of relative energy transfers, 0.0,-0.1,...-3.0.
*       anga(50) - array of log of scattering angles (rad), 0.4,0.3,...-4.5.
*       dang(50,30,61) - integral probability for muon scattering at an angle
*       larger than anga(i) for fixed values of ema(k) and va(j).
 
      em=emu
      t1=t0
      err=0
 
    9 em1=EM
      J=dlog10(em1)*10.+0.5+1
      IF(J.GE.71) J=71
 
C       IF THE MUON ENERGY BECOME LESS THAN 1 GEV THERE IS NO
C       SIMULATION OF FLUCTUATED ENERGY LOSSES
 
      IF(em1.LE.1.) GO TO 7
 
C       SIMULATION OF THE INTERACTION POINT
 
      yfl = ranum(1)
 
      FP=1./(CS2(J0,J,1)+CS2(J0,J,2)+CS2(J0,J,3)+CS2(J0,J,4))
      T=-FP*ALOG(YFL)
      T3=T1+T
      z1=T*dcos(theta)
 
   15 IF(z+z1.LE.zf) GO TO 10
 
*       If the next interaction point is deeper than the level of observation
*       we will calculate the continuous energy losses and multiple scattering
*       up to the level of observation
 
      IF(t3.GT.t10) THEN
        em1=mmu
        dr=t10
        go to 16
      END IF
      t2=(zf-z)/dabs(dcos(theta))
      dz=t2*dcos(theta)
 
C       CALCULATION OF THE CONTINUOUS ENERGY LOSSES:
C       DUE TO IONISATION AND OTHER PROCESSES FOR V=E/EMU<10**(-3)
 
      CALL vdem(em1,B0,BS)
      emf=em1-(em1*BS+B0)*T2
      IF(emf.LE.mmu) emf=mmu
      emi=10.**((dlog10(emf)+dlog10(em1))/2.)
      CALL vdem(emi,b0,bs)
      CALL vem(em1,b0,bs,t2,em3)
      IF(em3.LE.mmu) THEN
        dr=t1-t0+em1/b0
        em1=mmu
        go to 16
      END IF
      theta2=0.
      phi2=0.
      deltax=0.
      deltay=0.
      deltar=0.
      dx1=0.
      dy1=0.
      dz1=dz
 
*       simulation of the multiple scattering
      IF(ms_flag.EQ.1) THEN
        CALL multiple
     *(t2,em1,em3,b0,bs,theta2,phi2,deltax,deltay,deltar,err)
cc      IF (err.ne.0) WRITE (*,3057) err,em1,em3
 3057   FORMAT(1x,'%MUSIC-W, multiple error status ',i1,2(1x,f10.5),
     &    ' (standard call)')
      ENDIF
 
*       transformation of the coordinates
      CALL coord_transform(dx1,dy1,dz1,deltax,deltay,dz,theta,phi)
*       transformation of the angles
      IF(theta2.GT.0.) THEN
        CALL angle_transform(theta,phi,theta2,phi2)
      END IF
 
      em1=EM3
      IF(em3.LE.mmu) go to 16
      NM=1
      T=T-T2+deltar
      z=z+dz1
      x=x+dx1
      y=y+dy1
      t1=t1+t2+deltar
 
      GO TO 11
 
   10 CONTINUE
 
*       Calculation of the continuous energy losses of muon up to the next
*       interaction point
      CALL vdem(em1,B0,BS)
      emf=em1-(em1*BS+B0)*T
      IF(emf.LE.mmu) emf=mmu
      emi=10.**((dlog10(emf)+dlog10(em1))/2.)
      CALL vdem(emi,B0,BS)
      CALL vem(em1,B0,BS,T,EM2)
      IF(em2.LE.mmu) THEN
        dr=t1-t0+em1/b0
        em1=mmu
        go to 16
      END IF
      theta2=0.
      phi2=0.
      deltax=0.
      deltay=0.
      deltar=0.
      dx1=0.
      dy1=0.
      dz1=z1
 
      IF(ms_flag.EQ.1) THEN
        CALL multiple           !multiple scattering between 2 interactions
     *(t,em1,em2,b0,bs,theta2,phi2,deltax,deltay,deltar,err)
 3058   FORMAT(1x,'%MUSIC-W, multiple error status ',i1,2(1x,f10.5),
     &    ' (between two int. call)')
      END IF
 
*       transformation of the muon deflection to the base reference frame
      CALL coord_transform(dx1,dy1,dz1,deltax,deltay,z1,theta,phi)
 
      em1=em2
      IF(em1.LE.mmu) go to 16
      T1=T3+deltar
      z=z+dz1
      x=x+dx1
      y=y+dy1
 
      IF(theta2.GT.0.) THEN
*       transformation of the deflection angles to the base reference frame
        CALL angle_transform(theta,phi,theta2,phi2)
      END IF
 
      if(z.gt.zf) go to 11
      IF(em1.LE.1.) GO TO 7
 
      yfl = ranum(2)
      J=dlog10(em1)*10.+0.5+1
      IF(J.GE.71) J=71
 
C       SIMULATION OF THE TYPE OF MUON INTERACTION
      css=cs2(j0,j,1)+cs2(j0,j,2)+cs2(j0,j,3)+cs2(j0,j,4)
      al1=cs2(j0,j,1)/css
      al2=cs2(j0,j,2)/css
      al3=cs2(j0,j,3)/css
      IF(yfl.GT.AL1) GO TO 20
      ip=1
      GO TO 23
   20 IF(YFL.GT.AL1+AL2) GO TO 21
      IP=2
      GO TO 23
   21 IF(YFL.GT.AL1+AL2+AL3) GO TO 22
      IP=3
      GO TO 23
   22 IP=4
   23 CONTINUE
 
C       SIMULATION OF THE FRACTION OF MUON ENERGY TRANSFERED
C       TO THE SECONDARY PARTICLE
      yfl = ranum(3)
      CALL vcs(j,yfl,ip,v1)
 
      em=em1
c       simulation of the deflection angle of muon
      theta1=0.
      phi1=0.
      IF(ip.LE.3.AND.ip.GE.1.AND.d_flag.EQ.1) THEN
        CALL mu_scatt(ip,em,v1,theta1,phi1)
      END IF
 
      Demf=V1*em1
      EM=EM-Demf
      em1=em
c       transformation of the deflection angles to the base reference frame
      IF(theta1.GT.0.) THEN
        CALL angle_transform(theta,phi,theta1,phi1)
      END IF
 
      IF(em1.LE.mmu) go to 16
      GO TO 9
    7 CONTINUE
 
C       IF THE MUON ENERGY IS LESS THAN 1 GEV ONLY THE IONISATION
C       LOSSES ARE CALCULATED
      CALL vdem(em1,b0,bs)
      z4=zf-z
      t4=z4/dabs(dcos(theta))
      emf=em1-(em1*bs+b0)*t4
      IF(emf.LE.mmu) emf=mmu
      emi=10.**((dlog10(emf)+dlog10(em1))/2.)
      CALL vdem(emi,b0,bs)
      CALL vem(em1,b0,bs,t4,em3)
      IF(em3.LE.mmu) THEN
         dr=t1-t0+em1/b0
         em1=mmu
         go to 16
      END IF
      theta2=0.
      phi2=0.
      deltax=0.
      deltay=0.
      deltar=0.
      dx1=0.
      dy1=0.
      dz1=z4
 
*       simulation of the multiple scattering up to the level of observation
      IF(ms_flag.EQ.1) THEN
        CALL multiple
     *(t4,em1,em3,b0,bs,theta2,phi2,deltax,deltay,deltar,err)
 3059   FORMAT(1x,'%MUSIC-W, multiple error status ',i1,2(1x,f10.5),
     &    ' (after int. call)')
      END IF
 
*       transformation of the coordinates
      CALL coord_transform(dx1,dy1,dz1,deltax,deltay,z4,theta,phi)
 
      em1=em3
      IF(em1.LE.mmu) GO TO 16
      nm=2
      z=z+dz1
      x=x+dx1
      y=y+dy1
      t1=t1+t4+deltar
 
*       transformation of the angles
      IF(theta2.GT.0.) THEN
        CALL angle_transform(theta,phi,theta2,phi2)
      END IF
 
   11 CONTINUE
 
      dr=t1-t0
      emu_f=em1
 
   16 CONTINUE
 
      IF(em1.LE.mmu) emu_f=mmu
 
      RETURN
      END
 
*ENDFILE MEMBER=MUON_MUSIC
