*FILE MEMBER=muon_music_routines LIBRARY=SNOMAN LANGUAGE=FORTRAN77
*FILE DATE=15:Jan:2003
 
*     Contact:  J. Formaggio (UW)
 
*     Revision History:-
*     ================
*
*     4.02   J. Formaggio  First version.
*            N. West       Remove unused variables.
 
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      SUBROUTINE vcs(j,yfl,ip,v1)
C       CALCULATION OF THE FRACTION OF MUON ENERGY TRANSFERRED
C       TO THE SECONDARY PARTICLE
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      implicit double precision (a-h,o-z)
 
      include "cosmics.inc"
 
      real yfl
      i=1
    1 i=i+1
      if(yfl.gt.cs1(i,j,ip)) go to 1
      v3=(v2(i)-v2(i-1))/(cs1(i,j,ip)-cs1(i-1,j,ip))*
     *  (yfl-cs1(i-1,j,ip))+v2(i-1)
      v1=10.**v3
      RETURN
      END
 
 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      SUBROUTINE vdem(EM,B0,BS)
C       CALCULATION OF THE IONISATION LOSSES PER G/CM**2 AND
C       THE RELATIVE ENERGY LOSSES DUE TO OTHER PROCESSES FOR
C       V=E/EMU<10**(-3)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      implicit double precision (a-h,o-z)
      real f(81,6),em0(81)
      COMMON/coenlo/f,em0
 
C       EM0 - ARRAY OF MUON ENERGIES FOR WHICH THE RELATIVE ENERGY LOSSES
C               FOR V<10**(-3) HAVE BEEN ALREADY CALCULATED
 
C       F - ARRAY OF RELATIVE ENERGY LOSSES FOR V<10**(-3)
C       NOTE: IN THIS VARIANT OF PROGRAM THE ARRAY FOR RELATIVE
C       ENERGY LOSSES FOR V<10**(-3) IS USED
 
C       IF THE MUON ENERGY IS LESS THAN 1 GEV ONLY THE IONISATION
C       LOSSES ARE CALCULATED
 
C       CALCULATION OF THE RELATIVE ENERGY LOSSES DUE TO BREEMSTRAHLUNG,
C       PAIR PRODUCTION AND INELASTIC SCATTERING FOR V<10**(-3)
 
      e1=dlog10(em)
      i=1
    4 i=i+1
      if(i.gt.81) go to 6
      if(e1.gt.em0(i)) go to 4
    6 if(i.gt.81) i=81
      f1=(f(i,5)-f(i-1,5))/(em0(i)-em0(i-1))*(e1-em0(i-1))+f(i-1,5)
      bs=f1
      if(e1.le.0.) bs=0.
      f2=(f(i,6)-f(i-1,6))/(em0(i)-em0(i-1))*(e1-em0(i-1))+f(i-1,6)
      b0=f2
      IF(e1.LT.0.0.AND.e1.GT.-0.1) b0=f(10,6)
      IF(E1.LE.-0.9) B0=F(2,6)
 
      RETURN
      END
 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      SUBROUTINE vem(EM0,B0,BS,T,EM)
C       CALCULATION OF THE CONTINUOUS ENERGY LOSSES
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      implicit double precision (a-h,o-z)
      IF(bs*t.LE.1.e-6) GO TO 1
      em=em0*dexp(-bs*t)-b0/bs*(1.-dexp(-bs*t))
      return
    1 em=em0-b0*t-bs*t*em0
      RETURN
      END
 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      SUBROUTINE defl(em,ip,v,theta,phi)
*       Simulation of the muon scattering angles due to
*       inelastic scattering
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      implicit double precision (a-h,o-z)
 
      include "cosmics.inc"
 
      double precision ang(2,2)
      real yfl, ranum
      parameter (pi=3.141592654d0)
 
      IF(ip.EQ.3) THEN
        yfl = ranum(1)
        IF(em.LE.1.) RETURN
        eml=dlog10(em)
        k1=10.*eml+1.
        IF(k1.LT.1) k1=1
        IF(k1.GT.60) k1=60
        k2=k1+1
        IF(k2.LT.2) k2=2
        IF(k2.GT.61) k2=61
        vl=dlog10(v)
        m1=-vl*10.
        IF(m1.LT.1) m1=1
        IF(m1.GT.29) m1=29
        m2=m1+1
        IF(m2.LT.2) m2=2
        IF(m2.GT.30) m2=30
        DO kk=1,2
          IF(kk.EQ.1) k=k1
          IF(kk.EQ.2) k=k2
          DO mm=1,2
            IF(mm.EQ.1) m=m1
            IF(mm.EQ.2) m=m2
            j=1
    1       j=j+1
            IF(j.GT.49) THEN
              ang(mm,kk)=-5.
              go to 2
            END IF
            IF(yfl.GT.dang(j,m,k)) go to 1
            ang(mm,kk)=(anga(j)-anga(j-1))/(dang(j,m,k)-dang(j-1,m,k))*
     *        (yfl-dang(j-1,m,k))+anga(j-1)
    2       CONTINUE
          END DO
        END DO
        ang1=(ang(2,1)-ang(1,1))/(va(m2)-va(m1))*(vl-va(m1))+ang(1,1)
        ang2=(ang(2,2)-ang(1,2))/(va(m2)-va(m1))*(vl-va(m1))+ang(1,2)
        angt=(ang2-ang1)/(ema(k2)-ema(k1))*(eml-ema(k1))+ang1
cc        theta=10.**angt
        IF(angt.GT.-4.6) theta=10.**angt
        IF(angt.LE.-4.6) theta=0.
c        theta=180./pi*theta
	yfl = ranum(2)
        phi=2.*pi*yfl
        if(theta.eq.0.) phi=0.
      END IF
      RETURN
      END
 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      SUBROUTINE mu_scatt(jp,E,v,theta,phi)
      IMPLICIT NONE
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Input:
      INTEGER jp
!               * jp = 1 : bremsstrahlung
!               * jp = 2 : pair-production
!               * jp = 3 : nuclear
      double precision E    !initial energy of the muon
      double precision v        !fractional energy loss of muon
* Output:
      double precision theta,phi
      REAL yfl, ranum
* theta respect to parent muon direction,always positive!!!
* phi orthogonal to plane defined by parent muon direction
* theta phi in deg
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      double precision mmu,pi,mel,conv
      PARAMETER (pi=3.141592654)
      PARAMETER (conv=57.29577951)
      PARAMETER (mmu=105.655e-3)
      PARAMETER (mel=0.511e-3)
 
      include "cosmics.inc"

      real ee,vv
 
* Bremsstrahlung declarations
      double precision ak1,ak2,ak3,ak4,ak5,an
      double precision the1,the2,the3,the_mean,at
 
* Pair prod. declarations
      double precision a_min
 
* Bremsstrahlung: following Van Ginneken
 
      IF (jp.EQ.1) THEN
        ak1=0.092*E**(-1./3.)
        ak3=0.22*E**(-0.92)
        ak4=0.26*E**(-0.91)
        an=0.81*dsqrt(E)/(dsqrt(E)+1.8)
        ak2=0.052/E*n_z**(-0.25)
        at=min(ak1*dsqrt(v),ak2)
        the1=max(at,ak3*v)
        the2=ak4*v**(1.+an)*(1.-v)**(-an)
        ak5=ak4*0.5**(1.+an)*0.5**(-an)/0.5**(-0.5)
        the3=ak5/dsqrt(1.-v)
        IF(v.LE.0.5) the_mean=the1
        IF(v.GT.0.5.AND.the2.LT.0.2) the_mean=the2
        IF(v.GT.0.5.AND.the2.GE.0.2) the_mean=the3
        the_mean=the_mean*the_mean
   10   yfl = ranum(1)
        theta=-the_mean*alog(yfl)
        theta=dsqrt(theta)
        IF(theta.GT.pi) go to 10
        yfl = ranum(2)
        phi=(2*pi)*(yfl)                   !uniform between 0 and 2*pi
c        theta=theta*(conv)              !from rad to deg
c        phi=(phi)*(conv)
 
      ELSE IF (jp.EQ.2) THEN
        ee=E
        vv=v
        a_min=min(8.9e-4*sqrt(sqrt(vv))*(1.+1.5e-5*ee)+0.032*vv/
     *    (vv+1.),0.1)
        the_mean=(2.3+dlog(E))/E/(1.-v)*(v-2.*mel/E)**2/v/v*a_min
        the_mean=the_mean*the_mean
   11   yfl = ranum(3)
        theta=-the_mean*alog(yfl)
        theta=dsqrt(theta)
        IF(theta.GT.pi) go to 11
        yfl = ranum(4)
        phi=(2*pi)*(yfl)                   !uniform between 0 and 2*pi
c        theta=theta*(conv)              !from rad to deg
c        phi=(phi)*(conv)
 
* Inelastic scattering
 
      ELSE IF (jp.EQ.3) THEN
        CALL defl(E,jp,v,theta,phi)
 
      ELSE
        PRINT *,'Invalid jp argument on mu_scat',jp
        STOP
      ENDIF
      RETURN
      END
 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      SUBROUTINE multiple
     *(x0,einmu,eoutmu,alpha,beta,theta,phi,deltax,deltay,deltar,err)
      IMPLICIT NONE
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Multiple scattering: following
* P.Lipari and T. Stanev, Phys. Rev. D, 44, 3543 (1991) and
* L. Highland, Nucl. Instr. and Methods, 129, 497 (1975)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Input:
      double precision x0       !in km w.e.
      double precision einmu    !initial energy of the muon
      double precision eoutmu   !final energy of the muon
      double precision alpha    !ionization energy loss of muon
      double precision beta     !relative (continuous) energy losses of muon
                                !due to other processes
* Output:
      double precision theta
      double precision phi
      double precision deltax,deltay
      double precision deltar   !approximate increase of a muon path
      INTEGER err               !error flag (err=1) indicates that
                                !einmu or eoutmu is less than mmu
                                !error flag (err=2) indicates that
                                !integral along path is LE.0
 
* theta respect to parent muon direction
* phi orthogonal to plane defined by parent muon direction
* theta phi in deg
* deltax,deltay    lateral displacement of the muon in g/cm^2
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Multiple scattering declarations
      DOUBLE PRECISION mmu,pi,mel,conv
      PARAMETER (pi=3.141592654)
      PARAMETER (conv=57.29577951)
      PARAMETER (mmu=105.655e-3)
      PARAMETER (mel=0.511e-3)
 
 
      DOUBLE PRECISION ems,sigma,sigt,ro
      DOUBLE PRECISION sigmat,sigmax,thetax,thetay,zz,sz,step,sz1
      DOUBLE PRECISION p1,g1,bet1,p2,g2,bet2
      REAL sigmatt,sigmaxx,roo,thetaxx,thetayy,xxx,yyy
      REAL vv(2,2),cc(2,2),vcx(2)
 
C      PARAMETER (ems=0.0124)                 !in Gev (cfr Highland)
C      PARAMETER (ems=0.014)                  !in Gev (cfr Lipari)
      PARAMETER (ems=0.015)                  !in Gev (060697)
 
      INCLUDE "cosmics.inc"
      err=0
      IF(einmu.LE.mmu.OR.eoutmu.LE.mmu) THEN
        err=1
        RETURN
      END IF
      p1=dsqrt(einmu*einmu-mmu*mmu)
      g1=einmu/mmu
      bet1=dsqrt(g1*g1-1.)/g1
      p2=dsqrt(eoutmu*eoutmu-mmu*mmu)
      g2=eoutmu/mmu
      bet2=dsqrt(g2*g2-1.)/g2
 
      IF(beta.GT.0.) THEN
        sigt=1./alpha*(1./p2/bet2-1./p1/bet1)+beta/alpha/alpha*
     *    (dlog(dabs(1.+alpha/beta/p1/bet1))-
     *    dlog(dabs(1.+alpha/beta/p2/bet2)))
      END IF
      IF(beta.EQ.0.) THEN
        sigt=1./alpha*(1./p2/bet2-1./p1/bet1)
      END IF
      IF(sigt.LE.0.) THEN
        sigt=1./p1/bet1/p1/bet1
      END IF
      sigma=sigt*ems*ems/n_lambda
      sigmat=dsqrt(sigma)              !sigma(theta_x)=sigma(theta_y)
      IF(einmu.EQ.eoutmu) THEN                        !for test programs
        sigmat=dsqrt(ems*ems*x0/einmu/einmu/n_lambda)    !
      END IF						!
      sz=0.
      step=x0/100.
      zz=0.
    1 zz=zz+step/2.
      IF(beta.GT.0.) THEN
        sz1=p1*bet1*dexp(-beta*zz)-alpha/beta*(1.-dexp(-beta*zz))
      END IF
      IF(beta.EQ.0.) THEN
        sz1=p1*bet1-alpha*zz
      END IF
      IF(sz1.LE.mmu) THEN
        sz=0.
        go to 3
      END IF
      sz=sz+zz*zz*step/sz1/sz1
      zz=zz+step/2.
      IF(zz.LT.x0) go to 1
    3 CONTINUE
      IF(sz.LE.0.) THEN
        sz=x0*x0*x0/3./p1/bet1/p1/bet1
      END IF
      IF(sz.LE.0.) THEN
        err=2
        RETURN
      END IF
      sigmax=dsqrt(sz*ems*ems/n_lambda)        !sigma(x)=sigma(y)
      ro=sqrt(3.)/2.                  !correlation coefficient
    2 CONTINUE
      sigmatt=sigmat
      sigmaxx=sigmax
      roo=ro
c      CALL normco(thetaxx,xxx,0.,0.,sigmatt,sigmaxx,roo) !simulation of thetax,
c      CALL normco(thetayy,yyy,0.,0.,sigmatt,sigmaxx,roo) !thetay,xx,yy
	vv(1,1)=sigmatt*sigmatt
	vv(2,2)=sigmaxx*sigmaxx
	vv(1,2)=roo*sigmatt*sigmaxx
	vv(2,1)=roo*sigmatt*sigmaxx
	call corset(vv,cc,2)
	call corgen(cc,vcx,2)
	thetaxx=vcx(1)
	xxx=vcx(2)
	call corgen(cc,vcx,2)
	thetayy=vcx(1)
	yyy=vcx(2)
c      thetax=conv*thetaxx
c      thetay=conv*thetayy
      thetax=thetaxx
      thetay=thetayy
cc      IF(dabs(thetax).GE.pi/2..OR.dabs(thetay).GE.pi/2.) go to 2
      theta=datan(dsqrt(dtan(thetax)*dtan(thetax)+     !evaluation of theta
     *  dtan(thetay)*dtan(thetay)))                     !using thetax,thetay
      IF(dsin(thetax).NE.0.) THEN
        phi=datan(dtan(thetay)/dtan(thetax))            !evaluation of phi
        IF(dtan(thetax).LT.0.) phi=pi+phi             !using thetax,thetay
      END IF
      IF(dsin(thetax).EQ.0.) THEN
        phi=pi/2.
        IF(dtan(thetay).LT.0.) phi=pi*3./2.
      END IF
      IF(phi.GT.pi*2.) phi=-pi*2.+phi
      IF(phi.LT.0.) phi=phi+pi*2.
      deltax=xxx
      deltay=yyy
      deltar=sigmat*sigmat*x0
 
      RETURN
      END
 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	SUBROUTINE coord_transform(x,y,z,x1,y1,z1,theta1,phi1)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	IMPLICIT double precision (a-h,o-z)
	parameter (pi=3.141592654d0)
	phi2=phi1
	phi=phi2
	theta=theta1
	t11=dcos(theta)*dcos(phi)*dcos(phi)+dsin(phi)*dsin(phi)
	t21=dcos(theta)*dcos(phi)*dsin(phi)-dcos(phi)*dsin(phi)
	t31=-dsin(theta)*dcos(phi)
	t12=dcos(theta)*dcos(phi)*dsin(phi)-dcos(phi)*dsin(phi)
	t22=dcos(theta)*dsin(phi)*dsin(phi)+dcos(phi)*dcos(phi)
	t32=-dsin(theta)*dsin(phi)
	t13=dsin(theta)*dcos(phi)
	t23=dsin(theta)*dsin(phi)
	t33=dcos(theta)
	z=t31*x1+t32*y1+t33*z1
	x=t11*x1+t12*y1+t13*z1
	y=t21*x1+t22*y1+t23*z1
	RETURN
	END
 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	SUBROUTINE angle_transform(theta,phi,theta1,phi1)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	IMPLICIT double precision (a-h,o-z)
	parameter (pi=3.141592654d0)
	if(theta.eq.0.d0) then
	theta=theta1
	phi=phi1
	return
	end if
	t11=dcos(theta)*dcos(phi)*dcos(phi)+dsin(phi)*dsin(phi)
	t21=dcos(theta)*dcos(phi)*dsin(phi)-dcos(phi)*dsin(phi)
	t31=-dsin(theta)*dcos(phi)
	t12=dcos(theta)*dcos(phi)*dsin(phi)-dcos(phi)*dsin(phi)
	t22=dcos(theta)*dsin(phi)*dsin(phi)+dcos(phi)*dcos(phi)
	t32=-dsin(theta)*dsin(phi)
	t13=dsin(theta)*dcos(phi)
	t23=dsin(theta)*dsin(phi)
	t33=dcos(theta)
	x2=dsin(theta1)*dcos(phi1)
	y2=dsin(theta1)*dsin(phi1)
	z2=dcos(theta1)
	z21=t31*x2+t32*y2+t33*z2
	x21=t11*x2+t12*y2+t13*z2
	y21=t21*x2+t22*y2+t23*z2
	if(z21.gt.1.d0) z21=1.d0
	if(z21.lt.-1.d0) z21=-1.d0
	theta=dacos(z21)
	IF(x21.NE.0.d0) THEN
	phi=datan(y21/x21)
	IF(x21.LT.0.d0) phi=pi+phi
	END IF
	IF(x21.EQ.0.d0) THEN
	phi=pi/2.
	IF(y21.LT.0.d0) phi=1.5*pi
	END IF
	IF(phi.GT.2.*pi) phi=-2.*pi+phi
	IF(phi.LT.0.d0) phi=phi+2.*pi
	RETURN
	END
 
*ENDFILE MEMBER=muon_music_routines
