*FILE MEMBER=muon_transport LIBRARY=SNOMAN LANGUAGE=FORTRAN77
*FILE DATE=01:Jan:2002
*
      subroutine muon_transport(x0,y0,z0,cx0,cy0,cz0,Emuin0,depth0,tmu0)
*
*     SNOMAN: Calculate muon transport through rock for snoman
*
*
*     Contact:  J. Formaggio (UW)
*     Verified:
*     Refereed:
*
*     Specification:-
*     =============
*
*     Program Notes:-
*     =============
*
*     Revision History:-
*     ================
*
*     Argument Declarations:-
*     =====================
 
      implicit real*8 (a-h,o-z)
 
C     Include files
 
      INCLUDE "cosmics.inc"

*ENDHEADER
************************************************************************
 
      parameter (pi=3.141592654d0)
      emuin=emuin0
 
                                !input and output parameters:
        emu_f=0.                !final muon energy
 
        depth=depth0*100.       !depth along z-axis in g/cm^2, input
        path_max=1000000.*100.  !maximal path of muon, input
        emu=emuin               !initial muon energy, input
 
c        theta=0.               !angle with respect to z-axis, input and output
        phi=0.                  !angle with respect to x-axis, input and output
        dr=0.                   !muon path in the rock, output
        x=0.                    !deviation along the axes x,y, perpendicular
        y=0.                    !to z-axis, x,y-axes should be defined,
                                !input and output
        z=0.                    !z-coordinate
        t=0.                    !Initial muon path in rock, input
                                !emu_f - final energy of muon
 
c        theta0=theta
        emu_0=emu
        phi0=phi
        theta0=acos(cz0)
        theta=theta0
 
        CALL muon_music(emu,depth,emu_f,x,y,z,t,theta,phi,dr,path_max)
 
        gamma=(Emu_0+Emu_f)/2./0.105655
        beta=dsqrt(gamma*gamma-1.)/gamma
        vmu=beta*29.9792458
        tmu=dr/vmu/n_rho
        tmu0=tmu0+tmu
 
        thetac=dacos(cz0)
        IF(cx0.NE.0.) THEN
         phic=datan(cy0/cx0)
        IF(cx0.LT.0.) phic=pi+phic
        END IF
        IF(cx0.EQ.0.) THEN
        phic=pi/2.
        IF(cy0.LT.0.) phic=pi*3./2.
        END IF
        IF(phic.GT.pi*2.) phic=-pi*2.+phic
        IF(phic.LT.0.) phic=phic+pi*2.
 
        call coord_transform(x1,y1,z1,x,y,z,thetac,phic)
        x0=x1/n_rho+x0
        y0=y1/n_rho+y0
        z0=z1/n_rho+z0
        theta0=thetac
        phi0=phic
        call angle_transform(theta0,phi0,theta,phi)
        cz0=dcos(theta0)
        cx0=dsin(theta0)*dcos(phi0)
        cy0=dsin(theta0)*dsin(phi0)
        emuin0=emu_f                   !if emu_f is less than 0.106 the muon
                                       ! was stopped
        if(emuin0.le.0.106d0) emuin0=0.d0
 
        return
        end
 
      subroutine initialize_music
      implicit real*8 (a-h,o-z)

      include "cosmics.inc"

* Rock description
C      n_z=11.00       !Z, 11.00 for standard rock, 6.6 for pure water
C      n_a=22.00       !A, 22.00 for standard rock, 11.89 for pure water
C      n_rho=2.65      !density, 2.65 for standard rock
C      n_lambda=26.48  !rad. length, 26.48 for standard rock
C      ro=n_rho

*       reading of the muon energy losses
*      PRINT *,'music: loading energy losses table...'
      OPEN(UNIT=4,file='music-eloss-sno.dat',
     *form='formatted',status='old')
      READ(4,99)emulo
   99 FORMAT(16(5E12.5/),E12.5/)
      CLOSE(4)

*       reading of the arrays with the normalized 
*	(and integrated) double differential
*       cross-section of muon inelastic scattering
*       log10(E_mu(GeV))= 0.0 - 6.0 with a step of 0.1 (ema(61))
*       log10(v)= -0.1 - -3.0 with a step of 0.1 (va(30))
*       log10(theta(rad))= 0.4 - -4.5 with a step of 0.1 (anga(50))
*       normalized double differential cross-section dang(50,30,61)
*       was calculated using the program sign_ang_arr.for with
*       the subroutine sign_ang.for
*      PRINT *,'music: loading inel. scatt. cross section...'
      OPEN(UNIT=3,file='music-double-diff-rock.dat',
     *  FORM='formatted',STATUS='old')
      READ(3,122)ema,va,anga,dang
  122 FORMAT(5e15.7)
      CLOSE(3)

C       J0 - THE MINIMAL VALUE OF RELATIVE ENERGY LOSSES (E/EMU);
C               STARTING FROM THIS VALUE ALL KINDS OF MUON ENERGY
C               LOSSES EXCEPT FOR IONISATION ARE TREATED AS THE
C               THE FLUCTUATED ONES, FOR EXAMPLE: J0=201 MEANS
C               THAT FROM V=E/EMU=10**(-10) UP TO V=1 THE LOSSES
C               ARE TREATED AS THE FLUCTUATED ONES.
      J0=61

      DO I=1,100
        V2(I)=0.0-0.05*(I-1)
      END DO

      DO IL=1,81
        EMC(IL)=-1.0+0.1*(IL-1)
      END DO

C       READING THE INTEGRAL CROSS-SECTION
*      PRINT *,'music: loading integral cross sections...'
      OPEN(UNIT=1,file='music-cross-sections-sno.dat',
     *  FORM='formatted',STATUS='old')
      READ(1,3)CS2
    3 FORMAT(4(71(20(5E14.6/)/)/)/)
      CLOSE(UNIT=1)


C       FILLING THE ARRAY OF INTEGRAL CROSS-SECTIONS FOR THE SIMULATION

      do i=1,4
        do k=1,71
          do j=1,j0
             if(cs2(j0,k,i).gt.0.) then
      cs1(j,k,i)=cs2(j,k,i)/cs2(j0,k,i)
             end if
          end do
        end do
      end do

      return
      end

***********************************************************************
*endfile member = muon_transport
