*FILE MEMBER=music_cosmic LIBRARY=SNOMAN LANGUAGE=FORTRAN77
*FILE DATE=21:Jan:2004
*
      PROGRAM MUSIC_COSMIC
*
*             Calculate cosmic ray flux and throw event according
*             to saved energy-theta probability spectrum.
*
*
*     Contact:  J. Formaggio (UW)
*     Verified:
*     Refereed:
*
*     Parameters:
*     ==========
*
*     Common Block Access:-
*     ===================
*      None
*
*     Specification:-
*     =============
*     This code uses MUSIC to compute transport from surface to sno
*     It also stores the probabilities of the hanging wall and footwall
*     seperately (will be combined later).
*
*     Program Notes:-
*     =============
*
*     Revision History:-
*     ================
*
*     4.02   J. Formaggio  First version.
 
*
*     Argument Declarations:-
*     =====================
 
      IMPLICIT NONE
 
C     Include files
 
      INCLUDE 'cosmics.inc'

*ENDHEADER
************************************************************************
 
C     Local variables
 
      LOGICAL FIRST_CALL, THROW
      INTEGER I, K, N, ROCK
      INTEGER THROW_CALL, THROW_SAVE
      INTEGER THROW_EBIN, THROW_CTBIN
      INTEGER EBIN, CTBIN
      DOUBLE PRECISION THETA,PHI
      DOUBLE PRECISION X0,Y0,Z0,TTIME,DEPTH,LOGE,DE,DCT,DW
      DOUBLE PRECISION CX0,CY0,CZ0,PHI0,THETA0,EMU0
      DOUBLE PRECISION R, PROB, WEIGHT, ES0, ES1
      DOUBLE PRECISION EMUON, EN, NEUT_WT
      DOUBLE PRECISION WT_SUM, WT_E_SUM
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO = 0.D0)

      REAL MU_E, MU_DIR(3), MU_POS(3), RPROJ
      REAL VDOT, SNORAN, XV, LAMBDA, RPSUP, RP
      PARAMETER (RPSUP = 850.0)
 
      DATA FIRST_CALL /.TRUE./
      SAVE FIRST_CALL
 
      DATA THROW_SAVE /0/
      SAVE THROW_SAVE
 
C     If this is first call, run MUSIC to map muon cosmic ray
C     energy and angle to energy and angle at the SNO detector.
 
      CALL READ_PARAMETERS

      CALL INITIALIZE_MUSIC

      CALL NTCW(0)
  
      ROCK = 1
      WT_SUM   = ZERO
      WT_E_SUM = ZERO

      DO N = 1,NTHROWS
         X0 = 0.
         Y0 = 0.
         Z0 = 0.
         WEIGHT = 0.
            
         IF (MOD(N,1000).EQ.0) WRITE(6,*) 'PROCESSED ',N,' EVENTS.'

C     Find dI/dcos/dlog10(e) for muon flux
            
         CALL MUON_COSMIC_FLUX_SURFACE(EMU0,THETA0,PHI0,WEIGHT)
         
         CX0 = COS(PHI0)*SIN(THETA0)
         CY0 = SIN(PHI0)*SIN(THETA0)
         CZ0 = COS(THETA0)
         DEPTH = SNO_DEPTH * N_RHO ! Convert to m w.e.
         EMUON = EMU0
            
c     Use either parametrization or full calculation to find muon at sno
 
         CALL MUON_TRANSPORT(X0,Y0,Z0,CX0,CY0,CZ0,EMUON,DEPTH,TTIME)

         CALL MUON_SPALLATION(EMUON, EN, NEUT_WT)
         
C     Store in table for future use
            
         IF (EMUON.LE.0d0) GOTO 1001
         IF (CZ0.LE.VERTICAL) GOTO 1001

         WT_SUM = WT_SUM + WEIGHT / NTHROWS
         WT_E_SUM = WT_E_SUM + WEIGHT * EMUON / NTHROWS

         NT_ID = 1
         NT_EMU0   = REAL(EMU0)
         NT_EMU    = REAL(EMUON)
         NT_POS(1) = REAL(X0)
         NT_POS(2) = REAL(Y0)
         NT_POS(3) = REAL(Z0)
         NT_DIR(1) = REAL(CX0)
         NT_DIR(2) = REAL(CY0)
         NT_DIR(3) = REAL(CZ0)
         NT_TIME   = REAL(TTIME) 
         NT_WEIGHT = REAL(WEIGHT)
         NT_NWEIGHT= REAL(NEUT_WT)
         NT_NEUTE  = REAL(EN)

         CALL NTCW(1)

 1001    CONTINUE

      ENDDO
 
      CALL NTCW(2)

      WRITE(6,*) 'Estimated FLUX (g/cm-sq/s):',WT_SUM
      WRITE(6,*) 'Estimated mean energy:',WT_E_SUM/WT_SUM

      END
 
*ENDFILE MEMBER=music_cosmic
